using UnityEngine;
using System.Collections;

public class LSFCreepController : MonoBehaviour {

	private enum CreepState
	{
		Idle, // Wandering
		Following,
		Attacking,
		Dieing
	}

	private CreepState mState;

	[SerializeField]
	SpriteRenderer sprite;

	[SerializeField]
	SpriteRenderer innerSprite;
    
    [SerializeField]
    GameObject mRenderSprite;
	ISteerBehaviour mSteering;
	LSFPathfinder mPather;
    private Vector3 mLookDirection;

    // Pathing shits
    private GameObject mTargetToFollow;
    private const float kRepathDelay = 1.0f;
    private float mLastRepath = -1;

	[SerializeField]
	float attackDistanceSqr;

	[SerializeField]
	float attackTime;

	[SerializeField]
	int damage;

	bool canAttack = true;
	
	public int _team = -1;

	/// <summary>
	/// Used for tagging purposes and stuff
	/// </summary>
	public int originalTeam = -1;

	public int _target = -1;

	public int target
	{
		get 
		{
			return _target;
		}
		set
		{
			Debug.Log("Targetting " + value);
			_target = value;
			if (_target<0)
			{
				mState = CreepState.Idle;
			}
			else if (_target==team)
			{
                SetFollow(LSFPlayerManager.instance.players[team].gameObject);
			}
			else
			{
				mState = CreepState.Attacking;
			}
		}
	}

	public int team
	{
		get
		{
			return _team;
		}
		set
		{
			_team = value;
			if (_team < 0)
			{
				sprite.color = Color.white;
				innerSprite.color = Color.white;
			}
			else
			{
				sprite.color = LSFColors.colors[_team];
				innerSprite.color =  LSFColors.colors[originalTeam];
			}
		}
	}

	void Awake()
	{
		GetComponent<LSFLiving>().OnDeath += OnDeath;
	}

	// Use this for initialization
	void Start () {
		mSteering = null;
		mPather = GetComponent<LSFPathfinder> ();
	}

	void OnEnable()
	{
		LSFEventManager.Minions.OnPlayerDeath += OnPlayerDeath;
	}

	void OnDisable()
	{
		LSFEventManager.Minions.OnPlayerDeath -= OnPlayerDeath;
	}

	void OnDeath()
	{
		LSFEventManager.Minions.OnMinionDie(this);

		Destroy(gameObject);
	}
	
	public void MoveTo(Vector3 target)
	{
		mPather.PathTo (target);
	}

	public void MoveTo(GameObject target)
	{
		mPather.PathTo (target);
	}

	public void MoveTo(Transform trans)
	{
		MoveTo(trans.position);
	}

	// Update is called once per frame
	void Update () {
		switch (mState) {
		case CreepState.Idle:
			DoIdle();
			break;
		case CreepState.Following:
            DoFollow();
			break;
		case CreepState.Attacking:
            DoAttack();
			break;
		case CreepState.Dieing:
			break;
		}
	}

	IEnumerator Cooldown()
	{
		canAttack = false;
		yield return new WaitForSeconds(attackTime);
		canAttack = true;
	}

    public void SetFollow(GameObject followMe)
    {
        mState = CreepState.Following;
        mSteering = null;
        mTargetToFollow = followMe;
    }


    void OnTriggerEnter(Collider other)
    {
        Debug.Log("Collision, bitch!");
        if (team < 0)
        {
            LSFFlashlight flashlight = other.GetComponent<LSFFlashlight>();
            if (flashlight != null && flashlight.looker.CanHit(transform.position))
			{
				originalTeam = flashlight.playerIndex;
				team = flashlight.playerIndex;
				if (LSFEventManager.Minions.OnPlayerCaptureCreep != null)
				{
					LSFEventManager.Minions.OnPlayerCaptureCreep(this);
				}
            }
        }
    }

	void LateUpdate()
	{
		if(mSteering != null)
		{
            mLookDirection = mSteering.GetSteering();
			transform.Translate(mLookDirection * Time.deltaTime);
    	}
	}

	#region Update functions

	void DoIdle()
	{
        if (!(mSteering is LSFWander))
        {
            mSteering = new LSFWander();
        }

        mRenderSprite.transform.LookAt(mRenderSprite.transform.position + mLookDirection);

        // Draw the wandering direction
        Debug.DrawRay(transform.position, mLookDirection * 2.5f);
	}

    void DoFollow()
    {
        if (mTargetToFollow != null)
        {
            mRenderSprite.transform.LookAt(mTargetToFollow.transform.position);
            
            if (mLastRepath + kRepathDelay < Time.time)
            {
                mSteering = null;
                mPather.PathTo(mTargetToFollow.transform.position);
                mLastRepath = Time.time;
            }
        }
    }

    void DoAttack()
    {
        LSFCreepController creep = LSFMinionManager.instance.GetClosestMinionForPlayer(target, transform.position);
        Transform enemy = null;
        if (creep != null)
        {
            enemy = creep.transform;
        }
        else if (LSFPlayerManager.instance.players[target] != null)
        {
            enemy = LSFPlayerManager.instance.players[target].transform;
        }

        if (enemy != null)
        {
            mTargetToFollow = enemy.gameObject;
            DoFollow();
        }

        if (canAttack && (enemy.position - transform.position).sqrMagnitude < attackDistanceSqr)
        {
            enemy.GetComponent<LSFLiving>().TakeDamage(damage);
            StartCoroutine(Cooldown());
        }
    }

	#endregion

	void OnPlayerDeath(int index)
	{
		if (index == target && index != team)
		{
			target = -1;
			mState = CreepState.Idle;
		}
	}

#if UNITY_EDITOR
	void OnDrawGizmos()
	{

		TextGizmo.Draw (transform.position + new Vector3(0,0,2.0f), mState.ToString());

	}
#endif
}
