﻿using UnityEngine;
using System.Collections;

public class LSFLooker : MonoBehaviour {

	Vector3 currDir;

	public float baseRotation;

	public float batteryPower;

	[SerializeField]
	float batteryUseTime;

	[SerializeField]
	float batteryRechargeTime;

	[SerializeField]
	Transform lookField;

	[SerializeField]
	float minSqrMag;

	[SerializeField]
	Light spotLight;

	[SerializeField]
	float baseIntensity;

	[SerializeField]
	float maxDistance;

	public void Look (Vector3 dir)
	{
		currDir = dir;
	}

	public bool CanHit(Vector3 pos)
	{
		float dist = maxDistance * batteryPower;
		return (transform.position - pos).sqrMagnitude < dist * dist;
	}

	void LateUpdate()
	{
		if (currDir.sqrMagnitude > minSqrMag)
		{
			lookField.gameObject.SetActive(true);
			float rot = (Mathf.Atan2(currDir.y, currDir.x)) * Mathf.Rad2Deg + baseRotation;
			transform.localRotation = Quaternion.AngleAxis(-rot, new Vector3(0,1,0));
			currDir = Vector3.zero;
			batteryPower -= Time.deltaTime / batteryUseTime;
		}
		else
		{
			lookField.gameObject.SetActive(false);
			batteryPower += Time.deltaTime / batteryRechargeTime;
		}

		spotLight.intensity = baseIntensity * batteryPower;
		batteryPower = Mathf.Clamp01(batteryPower);
	}

	void OnDrawGizmos()
	{
		Gizmos.DrawLine(transform.position, transform.position + transform.forward * maxDistance * batteryPower);
	}
}
