﻿using UnityEngine;
using System.Collections;

public class LSFPlayer : MonoBehaviour {

	public int playerIndex;

	[HideInInspector]
	public LSFDimmer dimmer;

	[SerializeField]
	Light spotLight;

	[SerializeField]
	SpriteRenderer sprite;

	[SerializeField]
	float respawnTime;

	[SerializeField]
	float invulnOnNewLifeTime;

	[SerializeField]
	int creepsOnSpawn;

	Vector3 spawnPos;

	void Awake()
	{
		dimmer = GetComponent<LSFDimmer>();
        GetComponent<LSFLiving>().OnDeath += OnDeath;
		spawnPos = transform.localPosition;
	}

	void Start()
	{
		if (!LSFWhiteboard.playersActive[playerIndex])
		{
			Destroy(gameObject);
			return;
		}

		if (sprite != null) sprite.color = LSFColors.colors[playerIndex];
		if (spotLight != null) spotLight.color = LSFColors.colors[playerIndex];

		Init ();
	}

	void Init()
	{
		dimmer.bright = false;
		LSFPlayerManager.instance.players[playerIndex] = this;
		LSFCreepSpawner.instance.SpawnCreepsFromCP(transform, playerIndex, creepsOnSpawn);
		StartCoroutine(RunInvuln());
	}

	IEnumerator RunInvuln()
	{
		dimmer.invuln = true;
		yield return new WaitForSeconds(invulnOnNewLifeTime);
		dimmer.invuln = false;
	}

    void OnDeath()
    {

        if (LSFEventManager.Minions.OnPlayerDeath != null)
        {
            LSFEventManager.Minions.OnPlayerDeath(playerIndex);
        }

		StartCoroutine(DeathAndRespawn());
    }

	IEnumerator DeathAndRespawn()
	{
		LSFPlayerManager.instance.players[playerIndex] = null;
		transform.localPosition = new Vector3(100000,-100000,100000); //Get outta my sight, you miserable failure!
		yield return new WaitForSeconds(respawnTime);
		transform.localPosition = spawnPos;
		LSFPlayerManager.instance.players[playerIndex] = this;
		Init ();
	}

}
