﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class LSFMinionManager : MonoBehaviour {

	public static LSFMinionManager instance;

	List<LSFCreepController>[] playerCreeps = new List<LSFCreepController>[4];

	void Awake()
	{
		instance = this;
		Init ();
	}

	void Init()
	{
		for (int i = 0; i < playerCreeps.Length; i++)
		{
			playerCreeps[i] = new List<LSFCreepController>();
		}
	}

	void OnEnable()
	{
		LSFEventManager.Minions.OnMinionDie += OnCreepDeath;
		LSFEventManager.Minions.OnPlayerCaptureCreep += OnCreepCaptured;
		LSFEventManager.Minions.OnPlayerDeath += OnPlayerDeath;
		LSFEventManager.Minions.OnPlayerDim += OnPlayerDimmed;
		LSFEventManager.Minions.OnPlayerTagged += OnPlayerTagged;
	}

	void OnDisable()
	{
		LSFEventManager.Minions.OnMinionDie -= OnCreepDeath;
		LSFEventManager.Minions.OnPlayerCaptureCreep -= OnCreepCaptured;
		LSFEventManager.Minions.OnPlayerDeath -= OnPlayerDeath;
		LSFEventManager.Minions.OnPlayerDim -= OnPlayerDimmed;
		LSFEventManager.Minions.OnPlayerTagged -= OnPlayerTagged;
	}

	void OnCreepCaptured(LSFCreepController creep)
	{
		Debug.Log("Captured creep: " + creep.team);
		playerCreeps[creep.team].Add (creep);
	}

	void OnCreepDeath(LSFCreepController creep)
	{
		playerCreeps[creep.originalTeam].Remove(creep);
		if (!HasMinions(creep.originalTeam))
		{
			Debug.Log("No more minions!");
			LSFPlayerManager.instance.players[creep.team].dimmer.bright = true;
		}
	}

	void OnPlayerTagged(int tagged, int tagger)
	{
		Debug.Log(tagged + " tagged by " + tagger);
		foreach (var item in playerCreeps[tagged]) 
		{
			item.team = tagger;
			item.target = tagged;
		}
	}

	void OnPlayerDimmed(int index)
	{
		foreach (var item in playerCreeps[index]) 
		{
			item.team = index;
		}
	}

	void OnPlayerDeath(int index)
	{
		while(playerCreeps[index].Count > 0)
		{
			playerCreeps[index][0].team = -1;
			playerCreeps[index].RemoveAt(0);
		}
	}

	public void SetMinionTarget(int target, int targetter)
	{
		foreach (var item in playerCreeps[targetter]) 
		{
			if (item.team == targetter)
			{
				item.target = target;
			}
		}
	}

	[ContextMenu ("Log")]
	void LogCreeps()
	{
		Debug.Log ("Creeps per player:\n1: " + playerCreeps[0].Count + "\n2: " + playerCreeps[1].Count + "\n3: " + playerCreeps[2].Count + "\n4: " + playerCreeps[3].Count);
	}

	public bool HasMinions(int index)
	{
		return playerCreeps[index].Count > 0;
	}

	public LSFCreepController GetClosestMinionForPlayer(int index, Vector3 pos)
	{
		LSFCreepController closest = null;
		float closestDistance = 0;
		foreach (var item in playerCreeps[index]) 
		{
			if (item.team != index) continue;

			float dist = (item.transform.localPosition - pos).sqrMagnitude;
			if (closest == null || dist < closestDistance)
			{
				closestDistance = dist;
				closest = item;
			}
		}
		return closest;
	}

}
